ImplicitReplyObject subclass: #ConcurrentBoundedBuffer
	instanceVariableNames: 'array getIndex putIndex maxSize '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ex-BoundedBuffer-Ext'!


!ConcurrentBoundedBuffer methodsFor: 'initialize'!

initialize: anInteger
	maxSize := anInteger.
	array := Array new: maxSize + 1.
	getIndex := 1.
	putIndex := 1! !

!ConcurrentBoundedBuffer methodsFor: 'script'!

get
	| item |
	item := array at: getIndex.
	getIndex := (getIndex \\ array size) + 1.
	(Delay forSeconds: 2) wait.
	^item!

put: item
	array at: putIndex put: item.
	putIndex := (putIndex \\ array size) + 1.
	(Delay forSeconds: 2) wait! !

!ConcurrentBoundedBuffer methodsFor: 'printing'!

printOn: aStream

	super printOn: aStream.
	aStream nextPut: $(.
	getIndex <= putIndex
		ifTrue:
			[getIndex to: putIndex - 1 do:
				[:i |
				(array at: i) printOn: aStream.
				aStream nextPut: $ ]]
		ifFalse:
			[getIndex to: array size do:
				[:i |
				(array at: i) printOn: aStream.
				aStream nextPut: $ ].
			1 to: putIndex -1 do:
				[:i |
				(array at: i) printOn: aStream.
				aStream nextPut: $ ]].
	aStream nextPut: $)! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ConcurrentBoundedBuffer class
	instanceVariableNames: ''!


!ConcurrentBoundedBuffer class methodsFor: 'instance creation'!

new: size
	^self new initialize: size! !

!ConcurrentBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleSize: 3 numberItems: 8 speedRatio: 2"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 0) active.
	consumer := (Consumer new buffer: buffer delay: 2) active.
	producer runPut: numberItems.
	consumer runGet: numberItems! !

ConcurrentBoundedBuffer subclass: #GetRearConcurrentBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ex-BoundedBuffer-Ext'!


!GetRearConcurrentBoundedBuffer methodsFor: 'script'!

getRear
	putIndex := ((putIndex - 2) \\ (array size)) + 1.
	^array at: putIndex! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GetRearConcurrentBoundedBuffer class
	instanceVariableNames: ''!


!GetRearConcurrentBoundedBuffer class methodsFor: 'example'!

exampleGetRearSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGetRearSize: 3 numberItems: 8 speedRatio: 1"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 1) active.
	consumer := (Consumer new buffer: buffer delay: ratio) active.
	producer runPut: numberItems.
	consumer runGetRear: numberItems! !

ConcurrentBoundedBuffer subclass: #Get2ConcurrentBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ex-BoundedBuffer-Ext'!


!Get2ConcurrentBoundedBuffer methodsFor: 'script'!

get2
	^Array
		with: self get
		with: self get! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Get2ConcurrentBoundedBuffer class
	instanceVariableNames: ''!


!Get2ConcurrentBoundedBuffer class methodsFor: 'example'!

exampleGet2Size: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGet2Size: 3 numberItems: 8 speedRatio: 2"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 1) active.
	consumer := (Consumer new buffer: buffer delay: ratio) active.
	producer runPut: numberItems.
	consumer runGet2: numberItems! !

BoundedBuffer subclass: #IRBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ex-BoundedBuffer-Ext'!


!IRBoundedBuffer methodsFor: 'script'!

get
	"Remove and return the first item.
	Do not check buffer is empty."

	^contents removeFirst! !

!IRBoundedBuffer methodsFor: 'default classes'!

activityClass
	"A BoundedBuffer with implicit reply."

	^ImplicitReplyActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IRBoundedBuffer class
	instanceVariableNames: ''!


!IRBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleSize: 3 numberItems: 8 speedRatio: 4"
	"self exampleSize: 3 numberItems: 8 speedRatio: 1/4"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 1) active.
	consumer := (Consumer new buffer: buffer delay: ratio) active.
	producer runPut: numberItems.
	consumer runGet: numberItems! !

IRBoundedBuffer subclass: #Get2BoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ex-BoundedBuffer-Ext'!


!Get2BoundedBuffer methodsFor: 'script'!

get2
	^Array
		with: self get
		with: self get! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Get2BoundedBuffer class
	instanceVariableNames: ''!


!Get2BoundedBuffer class methodsFor: 'example'!

exampleGet2Size: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGet2Size: 3 numberItems: 8 speedRatio: 2"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 1) active.
	consumer := (Consumer new buffer: buffer delay: ratio) active.
	producer runPut: numberItems.
	consumer runGet2: numberItems! !

IRBoundedBuffer subclass: #GgetBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ex-BoundedBuffer-Ext'!


!GgetBoundedBuffer methodsFor: 'script'!

gget
	^self get! !

!GgetBoundedBuffer methodsFor: 'events'!

eventAccept: aMessage
	Transcript show: self printString , ' accept (' , aMessage compactPrintString , ')'; cr!

eventReceive: aMessage
	Transcript show: self printString , ' receive (' , aMessage compactPrintString , ')'; cr! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GgetBoundedBuffer class
	instanceVariableNames: ''!


!GgetBoundedBuffer class methodsFor: 'example'!

exampleGgetSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGgetSize: 3 numberItems: 8 speedRatio: 1"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 1) active.
	consumer := (Consumer new buffer: buffer delay: ratio) active.
	producer runPut: numberItems.
	consumer runGget: numberItems! !
