ExplicitAcceptObject subclass: #SimulatedBodyObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Body'!
SimulatedBodyObject comment:
'Class SimulatedBodyAcceptActorBehavior purpose is only to demonstrate that an active object with a body (autonomous live routine whose computation starts once the object is created) may be immediately simulated by an actor which sends to itself the body as a message as soon as created.
As a result this initial body message will be the first and only message to be implicitly accepted, as next ones will be explicitly accepted from within the body specification.

Class AcceptActorBehavior implements actor behaviors with a body.
The body is a live routine where the behavior explicitly specifies which sequence of which set of messages it is willing to accept. (See examples of body use in category Actalk-Ext-BodyAccept-Ex).
Computation of the body starts as soon as the object is created.

Class BodyAcceptActorBehavior is defined as a subclass of class AcceptActorBehavior to inherit explicit acceptance of messages which is the only way to accept messages (because there is no implicit acceptance of messages as for standard actors).

Class BodyAcceptActorBehavior corresponds roughly to POOL body and explicit message acceptance model except that message passing is of future type and not synchronous.
(However derivation of POOL type actors is instant in that they just need to implicitly wait onto the future returned. See classes PoolActor(Behavior)).

Acknowledgements:
See class PoolActorBehavior comment.'!


!SimulatedBodyObject methodsFor: 'activity creation'!

active
	"Send the message body to the newly created active object address."
	"Assume that the method bodyMessage does not specify a return value."

	^super active
		bodyMessage! !

ExplicitAcceptActivity subclass: #ObjectBodyActivity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Body'!


!ObjectBodyActivity methodsFor: 'activity setting'!

body
	"Delegate the specification of the body to the active object.
	This allows the active object to directly define the body."

	oself body! !
